//============================================================================
// Name        : NeptuneSampleApp.cpp
// Author      : imi-tech
// Version     :
// Copyright   : Your copyright notice
// Description : Hello World in C++, Ansi-style
// rm -f NeptuneSampleApp *.o *.d
//g++ -I/usr/local/include/neptune -O3 -Wall -c -fmessage-length=0 -fPIC -MMD -MP -MF NeptuneSampleApp.d -MT NeptuneSampleAPP.o -o NeptuneSampleApp.o NeptuneSampleApp.cpp
//g++ -L/usr/local/lib -o NeptuneSampleApp NeptuneSampleApp.o -lNeptuneAPI
//============================================================================

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <memory.h>


#include "NeptuneAPI.h"

#define	WORK_SIZE		1024

char	work_area[WORK_SIZE];
struct timeval 	time_start, time_end;
struct timeval result;

long long get_time()
{
	long long msec;
	struct timeval tv;

	gettimeofday(&tv, NULL);
	msec = tv.tv_usec/1000;
	msec += tv.tv_sec*1000;
	if( (msec%(10*60*1000)) == -1) return 0;
	else return (msec%(10*60*1000));
}

void callback(NImageFrame *pdata)
{
	static int nCount = 0;
	double 	et;

	if ( nCount == 0 )
	{
		gettimeofday(&time_start, NULL);
	}

	nCount++;
	if ( nCount == 31 ) {
		gettimeofday(&time_end, NULL);
		et = (time_end.tv_sec - time_start.tv_sec) + (time_end.tv_usec - time_start.tv_usec)/1000000.;
		printf(" fRate = %5.2lf (fps) \n", (double)(30.)/(et));
		nCount = 0;
	}

}

int main() {

	int nNum = 0;
	int nSel = -1;
	NErr status = eNeptuneSuccess;
	char strID[64] = {0,};
	NCamHandle hCam = NULL;

	NeptuneNumberOfCameras(&nNum);
	printf ("find camera : %d \n", nNum);
	for(int i = 0 ; i < nNum ; i++)
	{
		memset(strID, 0, 64);
		NeptuneGetCameraID(i, strID);
		printf("[%d] camera ID : %s \n", i, strID);
	}

	printf("---------------------------------------\n");

	printf("Select camera [0] to [%d] \n", nNum-1);

	while (1)
	{
		fgets(work_area, WORK_SIZE, stdin);
		nSel = atoi(work_area);
		if ( 0 <= nSel && nSel <= nNum-1 )
			break;
		else
			printf("Wrong camera index : %d \n", nSel);
	}

	if( (nNum > 0) && (nSel != -1) )
	{
		memset(strID, 0, 64);
		NeptuneGetCameraID(nSel, strID);
		status = NeptuneOpenCamera(strID, &hCam);
		if(status != eNeptuneSuccess) {
			printf("%s camera open fail!! %d \n", strID, status);
			return 0;
		}

		printf("%s camera open success!! %d \n", strID, status);

		int addr = 0xf2f10124;
		int data = 0;

		NeptuneReadRegister(hCam, addr, &data);
#if 1
		NCamInfo info;
		NSizeInfo sinfo;
		char strPixel[64] = {0,};

		memset(&info, 0, sizeof(NCamInfo));
		NeptuneGetCamInfo(hCam, &info);
		NeptuneGetSizeInfo(hCam, &sinfo);
		printf("camera info : %dx%d resolution, gain[%d], exposure[%f] \n", sinfo.width, sinfo.height, info.gain, info.exposure);

		NeptuneGetNumAvailablePixelFormats(hCam, &nNum);
		printf("available PixelFormats : \n");
		for(int i = 0 ; i < nNum ; i++)
		{
			NeptuneGetAvailablePixelFormatsIdxToStr(hCam, i, strPixel);
			printf(" - [%d]%s \n", i, strPixel);
		}

		printf("---------------------------------------\n");

		printf("Select pixel format [0] to [%d] \n", nNum-1);

		while (1)
		{
			fgets(work_area, WORK_SIZE, stdin);
			nSel = atoi(work_area);
			if ( 0 <= nSel && nSel <= nNum-1 )
				break;
			else
				printf("Wrong pixel format index : %d \n", nSel);
		}
		memset(strPixel, 0, 64);

		NeptuneGetAvailablePixelFormatsIdxToStr(hCam, nSel, strPixel);
		status = NeptuneSetPixelFormatStr(hCam, strPixel);
		printf(" set pixelformat[%s] %d\n", strPixel, status);

		printf("---------------------------------------\n");
		if(strstr(strPixel, "Bayer") != NULL) {
			printf(" NeptuneSetBayerConversion %d\n", eNeptuneBayerNearest);
			NeptuneSetBayerConversion(hCam, eNeptuneBayerNearest);
		}

		status =  NeptuneStartAcquisition(hCam, callback);
		if(status == eNeptuneSuccess)
		{
			printf("please press x button to stop\n");
			while(1)
			{
				if('x' == getchar())
					break;
				usleep(1000 * 10);
			}

			status = NeptuneStopAcquisition(hCam);
			printf("NeptuneStopAcquisition[%d] \n", status);
		}
		else
			printf("NeptuneStartAcquisition error[%d] \n", status);
#endif
		NeptuneCloseCamera(hCam);
		hCam = NULL;
	}
	return 0;
}
