//============================================================================
// Name        : NeptuneSampleApp.cpp
// Author      : imi-tech
// Version     :
// Copyright   : Your copyright notice
// Description : Hello World in C++, Ansi-style
//============================================================================

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <memory.h>

#include "NeptuneAPI.h"

#define	WORK_SIZE		1024

char	work_area[WORK_SIZE];
struct timeval 	time_start, time_end;
struct timeval result;

long long get_time()
{
	long long msec;
	struct timeval tv;

	gettimeofday(&tv, NULL);
	msec = tv.tv_usec/1000;
	msec += tv.tv_sec*1000;
	if( (msec%(10*60*1000)) == -1) return 0;
	else return (msec%(10*60*1000));
}


void callback(PNEPTUNE_IMAGE pImage, void* pContext)
{
    static int nCount = 0;
    double 	et;

    if ( nCount == 0 )
    {
        gettimeofday(&time_start, NULL);
    }

    nCount++;
    if ( nCount == 31 ) {
        gettimeofday(&time_end, NULL);
        et = (time_end.tv_sec - time_start.tv_sec) + (time_end.tv_usec - time_start.tv_usec)/1000000.;
        printf(" fRate = %5.2lf (fps) \n", (double)(30.)/(et));
        nCount = 0;
    }

    // timestamp
    //printf("%s : timestamp[%X]  \n", __FUNCTION__, pImage->uiTimestamp);
}

int main() {

    _uint32_t nNum = 0;
    int nSel = -1;
    ENeptuneError ret = NEPTUNE_ERR_Success;

    char strID[64] = {0,};
    NeptuneCamHandle hCam = NULL;

    ntcInit();

    ntcGetCameraCount(&nNum);
    printf ("find camera : %d \n", nNum);
    if(nNum < 1) {
        printf("Not find camera!!!\n");
        return 0;
    }

    PNEPTUNE_CAM_INFO pInfo = NULL;
    pInfo = new NEPTUNE_CAM_INFO[nNum];
    ntcGetCameraInfo(pInfo, nNum);
    for(int i = 0 ; i < nNum ; i++)
    {
        printf("[%d] camera ID : %s \n", i, pInfo[i].strCamID);
    }

    printf("---------------------------------------\n");

    printf("Select camera [0] to [%d] \n", nNum-1);

    while (1)
    {
        fgets(work_area, WORK_SIZE, stdin);
        nSel = atoi(work_area);
        if ( 0 <= nSel && nSel <= nNum-1 )
            break;
        else
            printf("Wrong camera index : %d \n", nSel);
    }

    if( (nNum > 0) && (nSel != -1) )
    {
        strcpy(strID, pInfo[nSel].strCamID);
        ret = ntcOpen(strID, &hCam);
        if(ret != NEPTUNE_ERR_Success) {
            printf("%s camera open fail!! %d \n", strID, ret);
            if(pInfo != NULL) {
                delete pInfo;
                pInfo = NULL;
            }
            return 0;
        }

        printf("%s camera open success!! %d \n", strID, ret);

        char strPixel[64] = {0,};
        NEPTUNE_IMAGE_SIZE sinfo;

        ntcGetImageSize(hCam, &sinfo);
        printf("camera info : %s-%s-%s %dx%d resolution \n", pInfo[nSel].strVendor, pInfo[nSel].strModel, pInfo[nSel].strIP, sinfo.nSizeX, sinfo.nSizeY);

        ENeptunePixelFormat* pPixelformat = NULL;
        _uint32_t nNumPixel = 0;

        ntcGetPixelFormatList(hCam, NULL, &nNumPixel);
        pPixelformat = new ENeptunePixelFormat[nNumPixel];

        ntcGetPixelFormatList(hCam, pPixelformat, &nNumPixel);
        printf("available PixelFormats[%d] : \n", nNumPixel);
        for(int i = 0 ; i < nNumPixel ; i++)
        {
            memset(strPixel, 0, 64);
            ntcGetPixelFormatString(hCam, pPixelformat[i], strPixel, 64);
            printf(" - [%d]%s \n", i, strPixel);
        }

        printf("---------------------------------------\n");

        printf("Select pixel format [0] to [%d] \n", nNumPixel-1);

        while (1)
        {
            fgets(work_area, WORK_SIZE, stdin);
            nSel = atoi(work_area);
            if ( 0 <= nSel && nSel <= nNumPixel-1 )
                break;
            else
                printf("Wrong pixel format index : %d \n", nSel);
        }
        memset(strPixel, 0, 64);
        ntcGetPixelFormatString(hCam, pPixelformat[nSel], strPixel, 64);
        ret = ntcSetPixelFormat(hCam, pPixelformat[nSel]);
        printf(" set pixelformat[%s] %d\n", strPixel, ret);

        printf("---------------------------------------\n");
        if(strstr(strPixel, "Bayer") != NULL) {
            printf(" ntcSetBayerConvert %d\n", NEPTUNE_BAYER_METHOD_NEAREST);
            ntcSetBayerConvert(hCam, NEPTUNE_BAYER_METHOD_NEAREST);
        }

        // if MTU is 9K, set Packet to 8000
        //ret = ntcSetPacketSize(hCam, 8000);

        ret = ntcSetFrameCallback(hCam, callback, NULL);
        ret = ntcSetAcquisition(hCam, NEPTUNE_BOOL_TRUE);
        if(ret == NEPTUNE_ERR_Success)
        {
            printf("please press x button to stop\n");
            while(1)
            {
                if('x' == getchar())
                    break;
                usleep(1000 * 10);
            }

            ret = ntcSetAcquisition(hCam, NEPTUNE_BOOL_FALSE);
            printf("ntcSetAcquisition[%d] \n", ret);
        }
        else
            printf("ntcSetAcquisition error[%d] \n", ret);

        ntcClose(hCam);
        hCam = NULL;

        if(pPixelformat != NULL) {
            delete pPixelformat;
            pPixelformat = NULL;
        }
    }

    if(pInfo != NULL) {
        delete pInfo;
        pInfo = NULL;
    }

    ntcUninit();
	return 0;
}
